unit Main;
{Freeware, P.J. Veger. Best, Netherlands}
interface
uses
	Classes, Controls, Dialogs, Forms, Graphics, Messages, Printers, StdCtrls,
	SysUtils, WinProcs, WinTypes, Mask;
type
	TSheet4Pages = class(TForm)
		PrinterSetupDlg: TPrinterSetupDialog;
		OpenDlg: TOpenDialog;
		PSetupBtn: TButton;
		PrintBtn: TButton;
		CloseBtn: TButton;
		TwoPagesCb: TCheckBox;
		NumberLinesCB: TCheckBox;
		Label1: TLabel;
		TabMsk: TMaskEdit;
		BothSidesCB: TCheckBox;
		procedure PSetupBtnClick(Sender: TObject);
		procedure PrintBtnClick(Sender: TObject);
		procedure CloseBtnClick(Sender: TObject);
	end;
var Sheet4Pages: TSheet4Pages;

implementation
{$R *.DFM}

procedure TSheet4Pages.PSetupBtnClick(Sender: TObject);
begin PrinterSetupDlg.Execute end;

procedure TSheet4Pages.CloseBtnClick(Sender: TObject);
begin Close end;

procedure TSheet4Pages.PrintBtnClick(Sender: TObject);
type TPPS=0..1;		{index range for number of 'pages' per side minus 1}
var HdrRgn: HRgn;
	PPS_1: TPPS;
	Rgn: array[TPPS] of HRgn; {one region per 'page'}
	X0: Integer;		{leftmost position; topmost position Y0 is 0}
	DX: Cardinal;		{Horizontal increment for 'page' switching}
	DY: Cardinal;		{Vertical increment for line switching}
	HTabs: array[0..0] of Word;
procedure PrintSides(const FName: string
							; Both: Boolean{True if printing at both sides}
							; Front: Boolean{True for front side of sheet}
							; NotLastSide: Boolean {True except for last side of last file});
	type AtEof=Exception;
	var F: TextFile;
		Buf: string;
		AltBuf: array[0..255] of char;
		PNum, LNum: Cardinal;
		I, X, Y: Cardinal;
	begin
		AssignFile(F, FName);
		Reset(F);
		if eof(F) then		{Print nothing at all for empty file}
			if Front then ShowMessage('The file '+FName+' is empty') else {skip}
		else
			try
				LNum:= 0;
				for PNum:= 1 to High(Cardinal) do begin
				{this page must actually be printed in three cases:
				1. if not sides, i.e. frontsides only, are used;
				2. if both sides are used and
					2.a we are printing frontsides and the page number is odd, or
					2.b we are printing back sides and the page number is even.
				Case 1 translates into a Boolean implication '<='; both cases
				2.a and 2.b are captured by the Boolean equivalence '='.
				We must go through all the motions when nothing has to be printed.
				Therefore, actual printing is guarded by the Boolean expression
				'Both<=(Front=odd(PNum))'}
					X:= X0;
					if Both<=(Front=odd(PNum)) then begin
						SelectClipRgn(Printer.Canvas.Handle, HdrRgn);
						Printer.Canvas.TextOut(X, 0
							, Lowercase(Format('Page %d: %s @ %s'
								, [PNum, FName
									, DateTimeToStr(FileDateToDateTime(FileAge(FName)))])))
					end;
					for I:= 0 to PPS_1 do begin
						SelectClipRgn(Printer.Canvas.Handle, Rgn[i]);
						Y := 2*DY;
						while Y<Printer.PageHeight do begin
							if eof(F) then raise AtEof.Create('');
							readln(F, Buf);
							inc(LNum);
							if Both<=(Front=odd(PNum)) then begin
								if NumberLinesCB.Checked then Buf:= Format('%4d ',[LNum])+Buf;
								TabbedTextOut(Printer.Handle, X, Y
									, StrPCopy(@AltBuf, Buf), Length(Buf), 1, HTabs, X);
							end;
							Inc(Y, DY)
						end;
						Inc(X,DX)
					end;
					if Both<=(Front=odd(PNum)) then Printer.NewPage;
				end
			except
				{At eof, when actually printing 'Both<=(Front=odd(PNum))', a NewPage
				has to be emitted. But this has also to be done in this special case:
				when printing 'Both' sides, when printing backsides 'Both and not Front',
				and eof occurs when emulating the printing of a frontside 'odd(PNum)'.
				This is the reason for the seconf implication in 'Both<=(Front<=odd(PNum))'.
				But note: apparently EndDoc sends Page Eject to Printer; for this reason
				'NotLastSide' is introduced}
				on AtEof do if Both<=(Front<=odd(PNum)) and NotLastSide then Printer.NewPage;
			end;
		CloseFile(F)
	end;
const Backsides = 'Wait.. and re-insert sheets for backside printing';
var I: Cardinal;
	PhysSize, PrintOffset : TPoint;
	Rect: TRect;
begin
	if not OpenDlg.Execute then exit;
	if TwoPagesCb.Checked then begin
		Printer.Orientation := poLandscape;
		PPS_1 := 1;
		Printer.Canvas.Font.Size := 7;
	end else begin
		Printer.Orientation := poPortrait;
		PPS_1 := 0;
		PRinter.Canvas.Font.Size := 9;
	end;
	Printer.Canvas.Font.Name := 'Courier New';
	DY := Printer.Canvas.TextHeight('0');
	I:= StrToInt(TabMsk.Text);
	if I=0 then I:= 1;
	HTabs[0]:= I*Printer.Canvas.TextWidth('0');
	Escape(Printer.Handle, GETPHYSPAGESIZE, 0, nil, @PhysSize);
	Escape(Printer.Handle, GETPRINTINGOFFSET, 0, nil, @PrintOffset);
	X0 := PhysSize.X - (Printer.PageWidth+2*PrintOffset.X);
	if X0>=0 then begin
		DX := Printer.PageWidth - X0;
	end else begin
		DX := Printer.PageWidth + X0;
		X0 := 0;
	end;
	HdrRgn:= CreateRectRgn(X0,0,X0+DX,2*DY);
	DX:= DX div (PPS_1+1);
	with Rect do begin
		Left:=X0-DX;
		Top:=2*DY;
		Right:= X0-2*Printer.Canvas.TextWidth('0');
		Bottom:=Printer.PageHeight
	end;
	for I:= 0 to PPS_1 do begin
		with Rect do begin
			Left:=Left+DX;
			Right:=Right+DX
		end;
		Rgn[I]:= CreateRectRgnIndirect(Rect)
	end;
	Printer.BeginDoc;
	try
		for I:= 0 to OpenDlg.Files.Count-1 do
			PrintSides(OpenDlg.Files.Strings[I], BothSidesCB.Checked, True, True);
		if BothSidesCB.Checked
		and (MessageDlg(Backsides, mtConfirmation, [mbOK, mbCancel], 0)=mrOK) then
			for I:= 0 to OpenDlg.Files.Count-1 do
				PrintSides(OpenDlg.Files.Strings[I], BothSidesCB.Checked, False, I<>OpenDlg.Files.Count-1)
		else begin
			Printer.Abort;
			WinProcs.AbortDoc(Printer.Canvas.Handle);
		end;
		Printer.EndDoc
	finally
		for I:=0 to PPS_1 do	DeleteObject(Rgn[I])
	end
end;

end.
